      program Hpair
c  map conf --> vpp is read from fort.12
c  single-particle Hamiltonian and pairing matrix elements in fort.14 format
c  Copyright 2007 by George F. Bertsch.  Use is permitted provided any
c  publications cite the code source, N. Sandulescu and G.F. Bertsch,
c  Phy. Rev. C 78 064318 (2008).
      include 'variablesb.h'
      real :: u(NCONF_MAX),scale
      integer Nlan
c
      read*, Nlan,scale

      call read_conf()
      call read_h(scale)
      call h_diag()
      print*, 'ihf,ehf',ihf,ehf
      u=0.0
      u(ihf)=1.0
      call lancz(u,Nlan)
      end
c
      subroutine read_conf()
      include 'variablesb.h'
      integer :: npair
      read(12,*) norb, npair, nconf
      do k=1,nconf
        read(12,*) (nocc(k,i),i=1,norb)
      enddo       
      do k = 1, NCONF_MAX**2
        read(12,*) iconf1(k),iconf2(k),iorb1(k),iorb2(k)
        if(iconf1(k).eq.-1) go to 99
      enddo
99    kmax=k-1
      end subroutine read_conf
c
      subroutine read_h(scale)  
      include 'variablesb.h'  
      real :: tmp,norb1,scale
      integer :: j1,j2,ix
      read(14,*) norb1
      if (norb1.ne.norb) stop "norb not consistent!"
      do i=1,norb
        read(14,*) ix,j1, esp(i)
      enddo
      do i1=1,norb
        do i2=1,i1
c warning:  fort.14 vpp may be off by factor of 2       warning
          read(14,*) j1,j2, vpp(i1,i2)
          vpp(i1,i2)=scale*vpp(i1,i2)  
          vpp(i2,i1)=vpp(i1,i2)
        enddo
      enddo 
      end subroutine read_h
c
      subroutine h_diag()
      include 'variablesb.h'
      real :: esum
      integer j
      ehf=1000.0
      do i=1,nconf
        esum=0.0
        do j=1,norb
          esum=esum+nocc(i,j)*esp(j)*2  
        enddo
        hdiag(i) = esum
        if (esum.lt.ehf) then
           ehf = esum
           ihf=i
        endif
      enddo

      end subroutine h_diag

c
      subroutine hpsi
      include 'variablesb.h'
      integer :: j1,j2
      htpsi=0.0
      do k=1,kmax        
        i1=iconf1(k)+1
        i2=iconf2(k)+1
        j1=iorb1(k)+1
        j2=iorb2(k)+1
        htpsi(i1) = htpsi(i1) +vpp(j1,j2)*tpsi(i2)
        htpsi(i2) = htpsi(i2) +vpp(j1,j2)*tpsi(i1)
      enddo
      do i=1,nconf
        htpsi(i) = htpsi(i) + hdiag(i)*tpsi(i)
      enddo
      end subroutine hpsi
c
      subroutine norm(snorm)
      include 'variablesb.h'
      real  :: snorm
      snorm=0.0
      do i=1,nconf
        snorm=snorm+tpsi(i)**2
      enddo
      snorm=sqrt(snorm)
      do i=1,nconf
        tpsi(i)=tpsi(i)/snorm
      enddo
      return
      end
c
      SUBROUTINE lancz(u,Nlan)
c  starting wave function is tpsi
c  hamiltonian multiplication is by hpsi
      include 'variablesb.h'
      real :: snorm, temp_psi(NCONF_MAX,20),psi(NCONF_MAX)
      real :: d(20),vv(20),a(20,20),u(NCONF_MAX),e0
      integer :: nlan
      do i=1,nconf
        tpsi(i)=u(i)
      enddo
        call norm(snorm)
        do i=1,nconf
          temp_psi(i,1)=tpsi(i)
        enddo 
        call hpsi
        d(1)=0.0
        do i=1,nconf
          d(1)=d(1)+temp_psi(i,1)*htpsi(i)
        enddo
        print*,'  d         vv '
        write(6,12) d(1)
        do k=2,Nlan
         do i=1,nconf
           tpsi(i)=htpsi(i)-d(k-1)*temp_psi(i,k-1)
         enddo
         if(k.gt.2) then
           do i=1,nconf
            tpsi(i)=tpsi(i)-vv(k-1)*temp_psi(i,k-2)
           enddo
         endif
         call norm(snorm)
         vv(k)=snorm
         do i=1,nconf
           temp_psi(i,k)=tpsi(i)
         enddo
         call hpsi
         d(k)=0.0
         do i=1,nconf
           d(k)=d(k)+temp_psi(i,k)*htpsi(i)
         enddo
         write(6,12) d(k),vv(k)
        enddo
         do i=1,nlan
           do k=1,nlan
             a(i,k)=0.0
           enddo
           a(i,i)=1.0
         enddo
	call tqli(d,vv,nlan,100,a)
        call eigsrt(d,a,nlan,100)
        print*,'energies'
	print*,(d(i),i=1,nlan)
        print*,'ecorr', d(nlan)-ehf
        close(6)
        stop
         do i=1,nconf
           psi(i) = 0.0
         enddo
         do k=1,nlan
           do i=1,nconf
            psi(i)=psi(i)+a(k,nlan)*temp_psi(i,k)
           enddo

         enddo
12       format(8f10.5)
      return
      end
c

      SUBROUTINE TRED2(A,N,NP,D,E)
      DIMENSION A(NP,NP),D(NP),E(NP)
      IF(N.GT.1)THEN
        DO 18 I=N,2,-1  
          L=I-1
          H=0.
          SCALE=0.
          IF(L.GT.1)THEN
            DO 11 K=1,L
              SCALE=SCALE+ABS(A(I,K))
11          CONTINUE
            IF(SCALE.EQ.0.)THEN
              E(I)=A(I,L)
            ELSE
              DO 12 K=1,L
                A(I,K)=A(I,K)/SCALE
                H=H+A(I,K)**2
12            CONTINUE
              F=A(I,L)
              G=-SIGN(SQRT(H),F)
              E(I)=SCALE*G
              H=H-F*G
              A(I,L)=F-G
              F=0.
              DO 15 J=1,L
                A(J,I)=A(I,J)/H
                G=0.
                DO 13 K=1,J
                  G=G+A(J,K)*A(I,K)
13              CONTINUE
                IF(L.GT.J)THEN
                  DO 14 K=J+1,L
                    G=G+A(K,J)*A(I,K)
14                CONTINUE
                ENDIF
                E(J)=G/H
                F=F+E(J)*A(I,J)
15            CONTINUE
              HH=F/(H+H)
              DO 17 J=1,L
                F=A(I,J)
                G=E(J)-HH*F
                E(J)=G
                DO 16 K=1,J
                  A(J,K)=A(J,K)-F*E(K)-G*A(I,K)
16              CONTINUE
17            CONTINUE
            ENDIF
          ELSE
            E(I)=A(I,L)
          ENDIF
          D(I)=H
18      CONTINUE
      ENDIF
      D(1)=0.
      E(1)=0.
      DO 23 I=1,N
        L=I-1
        IF(D(I).NE.0.)THEN
          DO 21 J=1,L
            G=0.
            DO 19 K=1,L
              G=G+A(I,K)*A(K,J)
19          CONTINUE
            DO 20 K=1,L
              A(K,J)=A(K,J)-G*A(K,I)
20          CONTINUE
21        CONTINUE
        ENDIF
        D(I)=A(I,I)
        A(I,I)=1.
        IF(L.GE.1)THEN
          DO 22 J=1,L
            A(I,J)=0.
            A(J,I)=0.
22        CONTINUE
        ENDIF
23    CONTINUE
      RETURN
      END
      SUBROUTINE TQLI(D,E,N,NP,Z)
      DIMENSION D(NP),E(NP),Z(NP,NP)
      IF (N.GT.1) THEN
        DO 11 I=2,N
          E(I-1)=E(I)
11      CONTINUE
        E(N)=0.
        DO 15 L=1,N
          ITER=0
1         DO 12 M=L,N-1
            DD=ABS(D(M))+ABS(D(M+1))
            IF (ABS(E(M))+DD.EQ.DD) GO TO 2
12        CONTINUE
          M=N
2         IF(M.NE.L)THEN
            IF(ITER.EQ.30)STOP 'too many iterations'
            ITER=ITER+1
            G=(D(L+1)-D(L))/(2.*E(L))
            R=SQRT(G**2+1.)
            G=D(M)-D(L)+E(L)/(G+SIGN(R,G))
            S=1.
            C=1.
            P=0.
            DO 14 I=M-1,L,-1
              F=S*E(I)
              B=C*E(I)
              IF(ABS(F).GE.ABS(G))THEN
                C=G/F
                R=SQRT(C**2+1.)
                E(I+1)=F*R
                S=1./R
                C=C*S
              ELSE
                S=F/G
                R=SQRT(S**2+1.)
                E(I+1)=G*R
                C=1./R  
                S=S*C
              ENDIF
              G=D(I+1)-P
              R=(D(I)-G)*S+2.*C*B
              P=S*R
              D(I+1)=G+P
              G=C*R-B
              DO 13 K=1,N

                F=Z(K,I+1)
                Z(K,I+1)=S*Z(K,I)+C*F
                Z(K,I)=C*Z(K,I)-S*F
13            CONTINUE
14          CONTINUE
            D(L)=D(L)-P
            E(L)=G
            E(M)=0.
            GO TO 1
          ENDIF
15      CONTINUE
      ENDIF
      RETURN
      END

      SUBROUTINE EIGSRT(D,V,N,NP)
      DIMENSION D(NP),V(NP,NP)
      DO 13 I=1,N-1
        K=I
        P=D(I)
        DO 11 J=I+1,N
          IF(D(J).GE.P)THEN
            K=J
            P=D(J)
          ENDIF
11      CONTINUE
        IF(K.NE.I)THEN
          D(K)=D(I)
          D(I)=P
          DO 12 J=1,N
            P=V(J,I)
            V(J,I)=V(J,K)
            V(J,K)=P
12        CONTINUE
        ENDIF
13    CONTINUE
      RETURN
      END
